function [PopDS,PopFS,ranks,flag,cdist]=IDEMO(Np,d,m,X,Xl,Xu,gen,Kop,type,obj)
% Syntax: [PopDS,PopFS,flag,cdist1]=IDEMO(Np,d,m,X,Xl,Xu,gen,Kop,type,obj)
% This function runs IDEMO with a initialised population.
% Input - 
%   Np: Scalar value defining no. of candidate solution in a population
%   d: Scalar value for the dimension of each encoded candidate vector 
%      (total number of decision variables)
%   m: Scalar value for total number of objectives
%   X: Parent population (initial or from previous population)
%   Xl: Row vector defining the lower bound of each decision variable
%   Xu: Row vector defining the upper bound of each decision variable
%   gen: Generation at which IDEMO is begining
%   Kop: #Generations for which IDEMO will be running at a stretch
%   type: Type of the test problem (DTLZ used in this work)
%   obj: indices of objectives in the objective set under consideration
% Output - 
%   PopDS: Population in decision space for the next generation 
%          (matrix order Np x d)
%   PopFS: Population in function/objective space for the next generation 
%          (matrix order Np x d)
%   ranks: ranks corresponding to each candidate in population
%          (matrix order Np x 2) col 1: non-dominated sorting, col 2: Dcomb
%   flag: set if new candidate is found, reset if no new candidate appears
%   cdist: value of Dcomb corresponding to each candidate

for i=gen:gen+Kop-1
    % Mutation
    V=mutation(X,Np,d,Xl,Xu);
    % Crossover
    U=crossover(X,V,Np,d,0.8); % CR = 0.8
    % New Selection and Ranking
    [PopDS,PopFS,ranks,flag,cdist]=SelRank(X,U,Np,d,m,type,obj);
    X=PopDS;
end
end