function obj=ObjRed(F,k,obj,pflag)
% Syntax: obj=ObjRed(F,k,obj,pflag)
% This function performs k-medoids clustering based objective reduction 
% with special care to singleton clusters.
% Input - 
%   F: Dataset in objective space
%   k: number of clusters
%   obj: vector of indices representing the input objective set
%   pflag: set flag to plot cluster index, see changes against iterations
% Output - 
%   obj: vector of indices representing the output objective set
%        (size of o/p obj <= size of i/p obj)

F1=F(:,obj); % select the subset of full objective set based on i/p obj
[IDX,~,seeds,pmdist,~,DistFull] = kmedoids(F1',k); % perform clustering
%% Plot performance if pflag is set
if pflag==1
    figure(k);
    X=1:size(F,2);
    Y=zeros(1,size(F,2));
    Y(obj)=IDX';
    stem(X,Y);
    xlabel('Objective Index');
    ylabel('Cluster Index');
end
%% Continue with objective reduction
temp = DistFull(seeds, :); % distance of medoids to all points
intermedoid = temp(:, seeds); % only medoid to medoid distances
count=zeros(1,k); % vector to hold #points in each cluster
flag=zeros(1,k); % flag for each cluster
for i=1:k
    count(i)=length(IDX(IDX==i)); % count is 1, if it is singleton
    if count(i) == 1
        [num, m1]=min(intermedoid(i,[1:i-1,i+1:end])); % D^i_{near}
        denom = max(pmdist(IDX == m1)); % D_{neigh}
        if num > (1.2 * denom) % if intermedoid distance is greater than twice the distance from farthest neighbor
            flag(i) = 1; % ignore such singleton cluster
        end
    end
end

minsum=2*k+10; % some large initial value
compclus=0;
for i=1:k % find most compact cluster
    compact=sum(pmdist(IDX==i));
    if compact<minsum && flag(i)~=1 % ignore singleton cluster with flag=1
        minsum=compact;
        compclus=i;
    end
end
if compclus==0 
    compclus=1;
end
objin=ones(1,length(obj)); % keep all objectives of the initial set
objin(IDX==compclus)=0;  % remove the most compact cluster
objin(seeds(compclus))=1; % restore the medoid of the compact cluster
objin=logical(objin);
obj=obj(objin);  % reduced obj set
end