function [X,F,cpop,crdist,idist,codist]=combrank(X,F,cpop,Np,obj,IdealP)
% Syntax: [X,F,cpop,crdist,idist,codist]=combrank(X,F,cpop,Np,obj,IdealP)
% This function combines the crowding distance and the distance from ideal 
% point to yield combined distance for ranking population.
% Input - 
%   X: Dataset in decision space
%   F: Dataset in objective space
%   cpop: indicators saying whether vectors are from new population or 
%         parent population (1: new population, 0: parent population)
%   Np: no. of candidates in population X
%   obj: vector of indices representing the chosen objective set
%   IdealP: ideal point for calculating distance from it
% Output - 
%   X: rearranged X by codist
%   F: rearranged F by codist 
%   cpop: rearranged cpop by codist
%   crdist: crowding distances corresponding to each candidate in X
%   idist: distance from IdealP corresponding to each candidate in X
%   codist: weighted combination of crdist and idist

crdist=zeros(Np,1); % Initialize for crowding distance
for j=1:length(obj)
    [~,In]=sort(F(:,obj(j))); % sort along j-th objective
    F=F(In,:);
    X=X(In,:);
    cpop=cpop(In,:);
    crdist=crdist(In,:);
    crdist(1,1)=1000; % highly weigh first and last candidate
    crdist(Np,1)=1000; % highly weigh first and last candidate
    fmin=F(1,obj(j));
    fmax=F(Np,obj(j));
    for i=2:Np-1 % crowding distance along rest of the solutions
        if fmax~=fmin
            crdist(i)=crdist(i)+abs(F(i+1,obj(j))-F(i-1,obj(j)))/(fmax-fmin);
        end
    end
end

w1 = X(:,end); % last column of population for adaptive weights
idist=pdist2(F(:,obj),IdealP(:,obj)); % ideal distance
codist=zeros(Np,1); % Initialize for combining distance
for i=1:Np
        codist(i)=w1(i,1)*(1/crdist(i))+(1-w1(i,1)).*idist(i);
end
    
[codist,ind2]=sort(codist,'ascend'); % sort by codist
% Rearrange F, X and cpop
X=X(ind2,:);
F=F(ind2,:);
cpop=cpop(ind2,:);
end