function [conv,Sample_Soln,fmax]=convergence(type,Soln)
% Syntax: [conv,Sample_Soln,fmax]=convergence(type,Soln)
% This function evaluates the convergence metric given the type of the
% problem and the rank-one solutions.
% Input - 
%   type: type of the DTLZ test problem
%   Soln: R1 solutions/ estimated Pareto-front (matrix order |PF| x #objs)
% Output - 
%   conv: Scalar value representing the convergence of normalized PF w.r.t.
%         true PF
%   Sample_Soln: Sampled set representing the true PF
%   fmax: reference vector for normalizing the estimated PF

num_objs=size(Soln,2);
Popsize=size(Soln,1);
N=500;
Sample_Soln=zeros(N,num_objs);  %To store the N sample points
conv=0;
switch type
    case 'DTLZ1'
        for i=1:N
            count=0;
            for j=1:num_objs
                rem=0.5-count;
                rand_num=rand()*rem;
                Sample_Soln(i,j)=rand_num;
                count=count+rand_num;
            end
            count=0.5-count;
            Sample_Soln(i,j)=Sample_Soln(i,j)+count;
        end
        TotalSet=vertcat(Soln,Sample_Soln);
        fmax=max(TotalSet);
        NormSet=repmat(fmax,N+1,1);
        for i=1:Popsize
            dtemp=[Soln(i,:) ; Sample_Soln];
            d=dtemp./NormSet;
            temp=pdist(d);
            D(i,1)=min(temp(1:N));
        end
        conv=sum(D);
    case 'DTLZ2'
        for i=1:N
            count=0;
            for j=1:num_objs-1
                rem=1-count;
                rand_num=rand()*rem;
                Sample_Soln(i,j)=sqrt(rand_num);
                count=count+rand_num;
            end
            count=1-count;
            Sample_Soln(i,num_objs)=Sample_Soln(i,num_objs)+sqrt(count);
        end
        TotalSet=vertcat(Soln,Sample_Soln);
        fmax=max(TotalSet);
        NormSet=repmat(fmax,N+1,1);
        for i=1:Popsize
            dtemp=[Soln(i,:) ; Sample_Soln];
            d=dtemp./NormSet;
            temp=pdist(d);
            D(i,1)=min(temp(1:N));
        end
        conv=sum(D);
    case 'DTLZ3'
        for i=1:N
            count=0;
            for j=1:num_objs-1
                rem=1-count;
                rand_num=rand()*rem;
                Sample_Soln(i,j)=sqrt(rand_num);
                count=count+rand_num;
            end
            count=1-count;
            Sample_Soln(i,num_objs)=Sample_Soln(i,num_objs)+sqrt(count);
        end
        TotalSet=vertcat(Soln,Sample_Soln);
        fmax=max(TotalSet);
        NormSet=repmat(fmax,N+1,1);
        for i=1:Popsize
            dtemp=[Soln(i,:) ; Sample_Soln];
            d=dtemp./NormSet;
            temp=pdist(d);
            D(i,1)=min(temp(1:N));
        end
        conv=sum(D);
    case 'DTLZ4'
        for i=1:N
            count=0;
            for j=1:num_objs-1
                rem=1-count;
                rand_num=rand()*rem;
                Sample_Soln(i,j)=sqrt(rand_num);
                count=count+rand_num;
            end
            count=1-count;
            Sample_Soln(i,num_objs)=Sample_Soln(i,num_objs)+sqrt(count);
        end
        TotalSet=vertcat(Soln,Sample_Soln);
        fmax=max(TotalSet);
        NormSet=repmat(fmax,N+1,1);
        for i=1:Popsize
            dtemp=[Soln(i,:) ; Sample_Soln];
            d=dtemp./NormSet;
            temp=pdist(d);
            D(i,1)=min(temp(1:N));
        end
        conv=sum(D);
    otherwise
        warning('Unexpected Function type. plese enter a valid choice');
end
conv=conv/Popsize;
end