function U=crossover(X,V,Np,d,CR)
% Syntax: U = crossover(X, V, Np, d, CR)
% This function performs the recombination stage of IDEMO.
% Input - 
%   X: Parent population at generation G (matrix order Np x d)
%   V: Donor population at generation G (matrix order Np x d)
%   Np: Population size (no. of rows of population matrix)
%   d: Dimension of candidate vectors (no. of columns of population matrix)
%   CR: cross-over rate, usually a high value in the range (0,1) to prefer
%       donor solutions 
% Output - 
%   U: Trial population for generation G+1 (matrix order Np x d)

U=zeros(Np,d); % Initialization
for i=1:Np
    Irand=randi([1,d],1); % random index to select element from donors 
    for j=1:d
        if j==Irand % for Irand, donor is always choosen
           U(i,j)=V(i,j); 
        else
            if rand(1)<=CR 
                U(i,j)=V(i,j); % select donor (CR)% of time
            else
                U(i,j)=X(i,j); % select parent rest of the time
            end
        end
    end
end
end