clear all
close all
clc

%% Initialize
M = input('Enter initial number of objectives:');
NP = 100;
type = input('Enter the type of benchmark function:'); 
% e.g. of type: 'DTLZ1', 'DTLZ2', 'DTLZ3', 'DTLZ4', 'DTLZ7' 
if strcmp(type(1:5),'DTLZ1')
    N = (M + 4) + 1; % last column is for adaptive weights
elseif strcmp(type(1:4),'DTLZ')
    N = (M + 9) + 1; % last column is for adaptive weights
else
    N = input('Enter number of decision variables:');
    N = N + 1; % last column is for adaptive weights
end
genmax=400;
gen=1;
k=input('Enter the number of objectives in reduced set:');
Kop=20;
Ksw=100;

% Initialization of Population
XL=zeros(1,N);
XU=ones(1,N);
X=initPop(NP,N,XL,XU);

obj=(1:M)'; % M objectives in full set
plotflag=0; % set flag to plot cluster index, see how it changes vs gen
flag=1; % will be reset if new candidate does not appear

%% Outer loop
h=waitbar(0,'Objective Reduction ...');
while gen<=genmax
    waitbar(gen/genmax);
    % Global search on full objective set
    [X,F,ranks,flag,cdist]=IDEMO(NP,N,M,X,XL,XU,gen,Kop,type,obj);
    gen=gen+Kop;
    waitbar(gen/genmax);
    if flag~=1
        break;
    end
    a=length(obj);
    if a<M % Roughly getting obj_a from obj_M
        Dist=1-corr(F(ranks(:,1)==1,:));
        for i=1:size(Dist)
            Dist(i,isnan(Dist(i,:)))=1;
            Dist(i,i)=0;
        end
        Distvec=sum(Dist,2);
        [~,obj]=sort(Distvec);
        obj=obj(M-a+1:M);
        obj=sort(obj);
    end
    %% Inner Loop
    while mod((gen-1),Ksw)~=0
        obj=ObjRed(F(ranks(:,1)==1,:),k,obj,plotflag);% Objective Reduction
        a=length(obj);
        % Local search on reduced objective set
        [X,F,ranks,flag,cdist]=IDEMO(NP,N,M,X,XL,XU,gen,Kop,type,obj);
        gen=gen+Kop;
        waitbar(gen/genmax);
        if flag~=1
            break;
        end
    end
    if flag==0 % If Inner Loop breaks, break from outer loop
       break; 
    end 
end
close(h);

%% Performance Analysis: Convergence and Hypervolume
ROS=F(ranks(:,1)==1,:); % Rank-One Solutions (ROS)
r=max(ROS); % reference point for hypervolume
hv=hypervolume(ROS,r); % Get dominated hypervolume
if strcmp(type(1:4),'DTLZ')
    [conv,Sample_Soln,fmax]=convergence(type,ROS); % Get convergence metric
    NormSet=repmat(fmax,size(ROS,1),1);
    ROS_norm = ROS./NormSet;
else
    disp('Information on true Pareto-front needed.');
    conv = 0;
end
%% Displaying Results
disp('=======');
disp('Results');
disp('=======');
fprintf(1,'Hypervolume: %0.4f\nConvergence: %0.4f\n',hv,conv);
fprintf(1,'Number of points in Pareto-Front (|PF|): %d\n',size(ROS,1));
fprintf(1,'No. of objectives in reduced set (m): %d\n',length(obj));
figure; parallelcoords(Sample_Soln);
xlabel('Objectives');ylabel('Objective Value');title('True Pareto-front');
figure; parallelcoords(ROS_norm);
xlabel('Objectives');ylabel('Objective Value');title('fDECOR Pareto-front');