function [IDX,C,seeds,pmdist,Cost,DistFull] = kmedoids(F,k)
% Syntax: [IDX,C,seeds,pmdist,Cost,DistFull] = kmedoids(F,k)
% This function performs k-medoid clustering using PAM (Partioning Around 
% Medoids) with correlation distance as the similarity measure.
% Input - 
%   F: Dataset in objective space (Columns: Objectives)
%   k: number of clusters
% Output - 
%   IDX: Vector of the clusters to which each row belongs
%   C: Medoids or the cluster centers
%   seeds: k observations (row numbers) which constitute the medoids
%   pmdist: point-to-medoid distance to corresponding medoids
%   Cost: Final (minimal) cost of performing PAM
%   DistFull: distance between all pairs of objectives

seeds=datasample(1:size(F,1),k,'Replace',false); % Select initial medoids
seeds=sort(seeds);
C=F(seeds,:);
DistFull=1-corr(F');
for i=1:size(DistFull)
    DistFull(i,isnan(DistFull(i,:)))=1;
    DistFull(i,i)=0;
end
        
% Associate each point to the closest medoid
Dist=DistFull(:,seeds); % Dist is of order size(F,1)-by-k
[pmdist,IDX]=min(Dist,[],2); 
Cost=sum(pmdist);

% Swap medoids and non-medoid data until minimum cost is attained
for m=1:k % for every medoids
    seeds1=seeds;
    for o=1:size(F,1) % for all data
        flag=1;
        for i=1:k % which is a non-medoid data
            if o==seeds(i) % is o a medoid data?
                flag=0;
                break;
            end
        end
        if flag==1 % i.e. o is a non-medoid data
            seeds1(m)=o;
            C1=F(seeds1,:);
            Dist1=DistFull(:,seeds1); % Dist is of order size(F,1)-by-k
            [pmdist1,IDX1]=min(Dist1,[],2); 
            Cost1=sum(pmdist1);
            if Cost1<Cost
                seeds=seeds1;
                C=C1;
                pmdist=pmdist1;
                IDX=IDX1;
                Cost=Cost1;
            end
        end
    end
end
end