function v=mutation(X,Np,d,Xl,Xu)
% Syntax: v = mutation(X, Np, d)
% This function performs the mutation stage of IDEMO.
% Input - 
%   X: Parent population at generation G (matrix order Np x d)
%   Np: Population size (no. of rows of population matrix)
%   d: Dimension of candidate vectors (no. of columns of population matrix)
%   Xl: Row vector defining the lower bound of each decision variable
%   Xu: Row vector defining the upper bound of each decision variable
% Output - 
%   v: Donor population at generation G (matrix order Np x d)

v=zeros(Np,d); % matrix initialization
for i=1:Np
    R=rand(1)*2; % R lies in [0,2] and is called Scale Factor
    % selecting 3 mutually exclusive indices excluding i
    tempvec=[1:i-1,i+1:Np];
    r=datasample(tempvec,3,'Replace',false);
    % performing mutation
    v(i,:)=X(r(1),:)+R.*(X(r(2),:)-X(r(3),:));
    % ensuring mutant vector is within the bounded decision space
    for j=1:d
        if v(i,j)>Xu(j)
            v(i,j)=Xu(j);
        elseif v(i,j)<Xl(j)
            v(i,j)=Xl(j);
        end
    end
end
end